<?php declare(strict_types = 1);
/*
** Zabbix
** Copyright (C) 2001-2023 Zabbix SIA
** Copyright (C) 2024 Cybertrust Japan Co., Ltd.
**
** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Affero General Public License for more details.
**
** You should have received a copy of the GNU Affero General Public License along with this program.
** If not, see <https://www.gnu.org/licenses/>.
**/


class CControllerSnapshotList extends CController {

	protected function init() {
		$this->disableCsrfValidation();
	}

	protected function checkInput(): bool {
		$fields = [
			'page'                => 'ge 1',
			'filter_set'          => 'in 1',
			'filter_rst'          => 'in 1',
			'filter_name'         => 'string',
			'sort'                => 'in name,created_at',
			'sortorder'           => 'in '.ZBX_SORT_DOWN.','.ZBX_SORT_UP,
			'uncheck'             => 'in 1'
		];

		$ret = $this->validateInput($fields);

		if (!$ret) {
			$this->setResponse(new CControllerResponseFatal());
		}

		return $ret;
	}

	protected function checkPermissions(): bool {
		switch ($this->getAction()) {
			case 'snapshot.host.list':
			case 'snapshot.template.list':
				return $this->checkSnapshotAccess();
			default:
				return false;
		}
	}

	protected function doAction(): void {
		if ($this->hasInput('filter_set')) {
			CProfile::update('web.snapshots.filter_name', $this->getInput('filter_name', ''), PROFILE_TYPE_STR);
		}
		elseif ($this->hasInput('filter_rst')) {
			CProfile::delete('web.snapshots.filter_name');
		}

		$filter = [
			'name' => CProfile::get('web.snapshots.filter_name', ''),
		];

		$sort_field = $this->getInput('sort', CProfile::get('web.snapshots.sort', 'name'));
		$sort_order = $this->getInput('sortorder', CProfile::get('web.snapshots.sortorder', ZBX_SORT_UP));

		CProfile::update('web.snapshots.sort', $sort_field, PROFILE_TYPE_STR);
		CProfile::update('web.snapshots.sortorder', $sort_order, PROFILE_TYPE_STR);

		$type = ZBX_SNAPSHOT_TYPE_NONE;
		switch ($this->getAction()) {
			case 'snapshot.host.list':
				$type = ZBX_SNAPSHOT_TYPE_HOST;
				break;
			case 'snapshot.template.list':
				$type = ZBX_SNAPSHOT_TYPE_TEMPLATE;
				break;
		}

		// Select snapshots.
		$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
		$query = 'select snapshotid,name,created_at,description,members from snapshots where type = '.$type;
		if ($filter['name'] != ''){
			$escaped_like = zbx_dbstr('%'.strtr($filter['name'], ['\\' => '\\\\', '%' => '\%', '_' => '\_']).'%');
			$query = $query.' and name like '.$escaped_like;
		}
		$query = $query.';';
		$snapshots = DBfetchArray(DBSelect($query));

		order_result($snapshots, $sort_field, $sort_order);

		if ($this->hasInput('page')) {
			$page_num = $this->getInput('page');
		}
		elseif (isRequestMethod('get')) {
			$page_num = 1;
		}
		else {
			$page_num = CPagerHelper::loadPage($this->getAction());
		}

		CPagerHelper::savePage($this->getAction(), $page_num);

		$paging = CPagerHelper::paginate($page_num, $snapshots, $sort_order,
			(new CUrl('zabbix.php'))->setArgument('action', $this->getAction())
		);

		$submenu = null;
		if ($this->checkSnapshotAccess()) {
			$items = [];

			$host_snapshot_url = (new CUrl('zabbix.php'))
				->setArgument('action', 'snapshot.host.list')
				->getUrl();
			$items[$host_snapshot_url] = _('Host snapshots');

			$template_snapshot_url = (new CUrl('zabbix.php'))
				->setArgument('action', 'snapshot.template.list')
				->getUrl();
			$items[$template_snapshot_url] = _('Template snapshots');

			$submenu = ['main_block' => ['items' => $items]];
		}

		$data = [
			'action' => $this->getAction(),
			'titleSubmenu' => $submenu,
			'snapshots' => $snapshots,
			'type' => $type,
			'paging' => $paging,
			'page' => $page_num,
			'filter' => $filter,
			'sortField' => $sort_field,
			'sortOrder' => $sort_order,
			'profileIdx' => 'web.snapshots.filter',
			'active_tab' => CProfile::get('web.snapshots.filter.active', 1),
			'config' => [
				'max_in_table' => CSettingsHelper::get(CSettingsHelper::MAX_IN_TABLE)
			],
			'allowed_ui_conf_templates' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES),
			'uncheck' => ($this->getInput('uncheck', 0) == 1)
		];

		$response = new CControllerResponseData($data);
		switch ($type) {
			case ZBX_SNAPSHOT_TYPE_HOST:
				$response->setTitle(_('Host snapshots'));
				break;
			case ZBX_SNAPSHOT_TYPE_TEMPLATE:
				$response->setTitle(_('Template snapshots'));
				break;
		}

		$this->setResponse($response);
	}
}
